==> cherry_flutter-0.0.3.tar.gz: lib/widgets/action_sheet/chf_action_sheet.dart <==
import 'package:flutter/material.dart';

import '../divider/chf_divider.dart';

class CHFActionSheet<T> {
  // 菜单选项
  final List<CHFActionSheetItem>? actions;

  // 顶部标题
  final String? title;

  // 取消按钮文字
  final String? cancelText;

  // 选项上方的描述信息
  final String? description;

  final Widget? customDescription;

  // 是否显示圆角
  final bool round;

  // 关闭图标
  final IconData? closeIcon;

  // 是否在点击遮罩层后关闭
  final bool closeOnClickOverlay;

  // 选中选项时触发，禁用或加载状态下不会触发
  final Function(
          CHFActionSheetItem actionSheetItem, int index, BuildContext context)?
      onSelect;

  // 取消按钮点击时触发
  final Function()? onCancel;

  // 关闭菜单时触发
  final Function()? onClose;

  // 自定义菜单内容
  final Widget? child;
  final double? roundRadius;

  // 背景色
  final Color? background;
  final Color? lineColor;
  final Color? spacerColor;
  final Color? cancelColor;

  const CHFActionSheet({
    Key? key,
    this.actions,
    this.title,
    this.cancelText,
    this.description,
    this.customDescription,
    this.round: true,
    this.roundRadius: 0.0,
    this.closeIcon,
    this.closeOnClickOverlay: true,
    this.onSelect,
    this.onCancel,
    this.onClose,
    this.child,
    this.background: Colors.white,
    this.lineColor,
    this.spacerColor,
    this.cancelColor,
  });

  show(BuildContext context) {
    return showModalBottomSheet<T>(
        context: context,
        backgroundColor: background,
        isDismissible: closeOnClickOverlay,
        isScrollControlled: true,
        shape: RoundedRectangleBorder(
            borderRadius:
                BorderRadius.vertical(top: Radius.circular(roundRadius!))),
        builder: (BuildContext context) {
          return CHFActionSheetState<T>(this);
        });
  }
}

class CHFActionSheetState<T> extends StatelessWidget {
  final CHFActionSheet actionSheet;

  const CHFActionSheetState(this.actionSheet);

  List<Widget> buildActionSheetItemContent(CHFActionSheetItem action) {
    return [
      action.loading
          ? Container(
              width: 16.0,
              height: 16.0,
              margin: EdgeInsets.symmetric(horizontal: 5),
              child: CircularProgressIndicator(
                valueColor: AlwaysStoppedAnimation(Color(0xff323233)),
                backgroundColor: Colors.white,
                strokeWidth: 1,
              ),
            )
          : Container(),
      Text(action.name ?? "",
          style: TextStyle(
              fontSize: 16,
              color: action.disabled
                  ? Color(0xffc8c9cc)
                  : action.color ?? Color(0xff323233))),
      SizedBox(width: action.subName != null ? 6 : 0),
      Text(action.subName ?? "",
          style: TextStyle(fontSize: 12, color: Color(0xff646566)))
    ];
  }

  buildActionSheetItem(BuildContext context, List<CHFActionSheetItem> actions) {
    List<Widget> widgets = [];
    if (actionSheet.child != null) return [actionSheet.child];
    for (int i = 0; i < actions.length; i++) {
      CHFActionSheetItem action = actions[i];
      widgets.add(Column(
        children: <Widget>[
          Material(
            type: MaterialType.transparency,
            child: InkWell(
              focusColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).focusColor,
              highlightColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).highlightColor,
              hoverColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).hoverColor,
              splashColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).splashColor,
              child: Container(
                height: 50,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: buildActionSheetItemContent(action),
                ),
              ),
              onTap: () {
                if (action.loading || action.disabled) return;
                if (actionSheet.onSelect != null)
                  actionSheet.onSelect!(action, i, context);
                close(context, action.value);
              },
            ),
          ),
          i < actions.length
              ? CHFDivider(hairline: true, lineColor: actionSheet.lineColor)
              : Container()
        ],
      ));
    }
    return widgets;
  }

  Widget buildTitleItem(BuildContext context) {
    return Column(
      children: <Widget>[
        Stack(
          children: <Widget>[
            Container(
              alignment: AlignmentDirectional.center,
              padding: EdgeInsets.symmetric(horizontal: 20, vertical: 10),
              child: Column(
                children: <Widget>[
                  actionSheet.title != null
                      ? Text(actionSheet.title!,
                          style: TextStyle(
                              fontSize: 16, fontWeight: FontWeight.bold))
                      : Container(),
                  SizedBox(height: 4),
                  if (actionSheet.customDescription != null)
                    actionSheet.customDescription!,
                  if (actionSheet.description != null)
                    Text(actionSheet.description!,
                        style: TextStyle(
                            fontSize: 11, color: const Color(0xffAAAAAA)))
                ],
              ),
            ),
            Positioned(
              right: 0,
              top: 0,
              child: Padding(
                padding: EdgeInsets.all(12),
                child: GestureDetector(
                  onTap: () {
                    close(context);
                  },
                  child: Icon(actionSheet.closeIcon,
                      color: Color(0xff969799), size: 18),
                ),
              ),
            )
          ],
        ),
        CHFDivider(hairline: true, lineColor: actionSheet.lineColor)
      ],
    );
  }

  Widget buildCancelItem(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        Container(
            height: 8, color: actionSheet.spacerColor ?? Color(0xfff7f8fa)),
        DecoratedBox(
          decoration: BoxDecoration(
//            color: Colors.white,
              ),
          child: Material(
            type: MaterialType.transparency,
            child: InkWell(
              child: Container(
                alignment: AlignmentDirectional.center,
                height: 50,
                child: Text(actionSheet.cancelText!,
                    style: TextStyle(
                        fontSize: 16,
                        color: actionSheet.cancelColor ?? Color(0xff323233))),
              ),
              onTap: () {
                if (actionSheet.onCancel != null) actionSheet.onCancel!();
                close(context);
              },
            ),
          ),
        ),
      ],
    );
  }

  void close(BuildContext context, [T? value]) {
    if (actionSheet.onClose != null) actionSheet.onClose!();
    if (value != null) {
      Navigator.of(context).pop<T>(value);
    } else {
      Navigator.of(context).pop<T>(value);
    }
  }

  @override
  Widget build(BuildContext context) {
    return ClipRRect(
      borderRadius: BorderRadius.vertical(
        top: Radius.circular(actionSheet.roundRadius!),
      ),
      child: Container(
//        decoration: BoxDecoration(
//          color: Colors.white,
//            borderRadius: BorderRadius.vertical(
//                top: Radius.circular(actionSheet.round ? 20 : 0)),
//            ),
          child: SafeArea(
              child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          (actionSheet.title != null || actionSheet.description != null)
              ? buildTitleItem(context)
              : Container(),
          ...buildActionSheetItem(context, actionSheet.actions!),
          (actionSheet.cancelText != null)
              ? buildCancelItem(context)
              : Container()
        ],
      ))),
    );
  }
}

class CHFActionSheetItem<T> {
  // 标题
  final String? name;

  // 二级标题
  final String? subName;

  // 选项文字颜色
  final Color? color;

  // 是否为加载状态
  final bool loading;

  // 是否为禁用状态
  final bool disabled;

  // 选项值
  final T? value;

  CHFActionSheetItem({
    this.name,
    this.subName,
    this.color,
    this.loading: false,
    this.disabled: false,
    this.value,
  });
}

==> cherry_flutter-0.0.6.tar.gz: lib/widgets/action_sheet/chf_action_sheet.dart <==
import 'package:flutter/material.dart';

import '../divider/chf_divider.dart';

class CHFActionSheet<T> {
  // 菜单选项
  final List<CHFActionSheetItem>? actions;

  // 顶部标题
  final Widget? title;

  // 取消按钮文字
  final String? cancelText;

  // 选项上方的描述信息
  final String? description;

  final Widget? customDescription;

  // 是否显示圆角
  final bool round;

  // 关闭图标
  final IconData? closeIcon;

  // 是否在点击遮罩层后关闭
  final bool closeOnClickOverlay;

  // 选中选项时触发，禁用或加载状态下不会触发
  final Function(
          CHFActionSheetItem actionSheetItem, int index, BuildContext context)?
      onSelect;

  // 取消按钮点击时触发
  final Function()? onCancel;

  // 关闭菜单时触发
  final Function()? onClose;

  // 自定义菜单内容
  final Widget? child;
  final double? roundRadius;

  // 背景色
  final Color? background;
  final Color? lineColor;
  final Color? spacerColor;
  final Color? cancelColor;

  const CHFActionSheet({
    Key? key,
    this.actions,
    this.title,
    this.cancelText,
    this.description,
    this.customDescription,
    this.round: true,
    this.roundRadius: 0.0,
    this.closeIcon,
    this.closeOnClickOverlay: true,
    this.onSelect,
    this.onCancel,
    this.onClose,
    this.child,
    this.background: Colors.white,
    this.lineColor,
    this.spacerColor,
    this.cancelColor,
  });

  show(BuildContext context) {
    return showModalBottomSheet<T>(
        context: context,
        backgroundColor: background,
        isDismissible: closeOnClickOverlay,
        isScrollControlled: true,
        shape: RoundedRectangleBorder(
            borderRadius:
                BorderRadius.vertical(top: Radius.circular(roundRadius!))),
        builder: (BuildContext context) {
          return CHFActionSheetState<T>(this);
        });
  }
}

class CHFActionSheetState<T> extends StatelessWidget {
  final CHFActionSheet actionSheet;

  const CHFActionSheetState(this.actionSheet);

  List<Widget> buildActionSheetItemContent(CHFActionSheetItem action) {
    return [
      action.loading
          ? Container(
              width: 16.0,
              height: 16.0,
              margin: EdgeInsets.symmetric(horizontal: 5),
              child: CircularProgressIndicator(
                valueColor: AlwaysStoppedAnimation(Color(0xff323233)),
                backgroundColor: Colors.white,
                strokeWidth: 1,
              ),
            )
          : Container(),
      Text(action.name ?? "",
          style: TextStyle(
              fontSize: 16,
              color: action.disabled
                  ? Color(0xffc8c9cc)
                  : action.color ?? Color(0xff323233))),
      SizedBox(width: action.subName != null ? 6 : 0),
      Text(action.subName ?? "",
          style: TextStyle(fontSize: 12, color: Color(0xff646566)))
    ];
  }

  buildActionSheetItem(BuildContext context, List<CHFActionSheetItem> actions) {
    List<Widget> widgets = [];
    if (actionSheet.child != null) return [actionSheet.child];
    for (int i = 0; i < actions.length; i++) {
      CHFActionSheetItem action = actions[i];
      widgets.add(Column(
        children: <Widget>[
          Material(
            type: MaterialType.transparency,
            child: InkWell(
              focusColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).focusColor,
              highlightColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).highlightColor,
              hoverColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).hoverColor,
              splashColor: (action.disabled || action.loading)
                  ? Colors.transparent
                  : Theme.of(context).splashColor,
              child: Container(
                height: 50,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: buildActionSheetItemContent(action),
                ),
              ),
              onTap: () {
                if (action.loading || action.disabled) return;
                if (actionSheet.onSelect != null)
                  actionSheet.onSelect!(action, i, context);
                close(context, action.value);
              },
            ),
          ),
          i < actions.length
              ? CHFDivider(hairline: true, lineColor: actionSheet.lineColor)
              : Container()
        ],
      ));
    }
    return widgets;
  }

  Widget buildTitleItem(BuildContext context) {
    return Column(
      children: <Widget>[
        Stack(
          children: <Widget>[
            Container(
              alignment: AlignmentDirectional.center,
              padding: EdgeInsets.symmetric(horizontal: 20, vertical: 10),
              child: Column(
                children: <Widget>[
                  if(actionSheet.title != null)
                    actionSheet.title!,
                  if (actionSheet.customDescription != null)
                    actionSheet.customDescription!,
                  if (actionSheet.description != null)
                    Text(actionSheet.description!,
                        style: TextStyle(
                            fontSize: 11, color: const Color(0xffAAAAAA)))
                ],
              ),
            ),
            Positioned(
              right: 0,
              top: 0,
              child: Padding(
                padding: EdgeInsets.all(12),
                child: GestureDetector(
                  onTap: () {
                    close(context);
                  },
                  child: Icon(actionSheet.closeIcon,
                      color: Color(0xff969799), size: 18),
                ),
              ),
            )
          ],
        ),
        CHFDivider(hairline: true, lineColor: actionSheet.lineColor)
      ],
    );
  }

  Widget buildCancelItem(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        Container(
            height: 8, color: actionSheet.spacerColor ?? Color(0xfff7f8fa)),
        DecoratedBox(
          decoration: BoxDecoration(
//            color: Colors.white,
              ),
          child: Material(
            type: MaterialType.transparency,
            child: InkWell(
              child: Container(
                alignment: AlignmentDirectional.center,
                height: 50,
                child: Text(actionSheet.cancelText!,
                    style: TextStyle(
                        fontSize: 16,
                        color: actionSheet.cancelColor ?? Color(0xff323233))),
              ),
              onTap: () {
                if (actionSheet.onCancel != null) actionSheet.onCancel!();
                close(context);
              },
            ),
          ),
        ),
      ],
    );
  }

  void close(BuildContext context, [T? value]) {
    if (actionSheet.onClose != null) actionSheet.onClose!();
    if (value != null) {
      Navigator.of(context).pop<T>(value);
    } else {
      Navigator.of(context).pop<T>(value);
    }
  }

  @override
  Widget build(BuildContext context) {
    return ClipRRect(
      borderRadius: BorderRadius.vertical(
        top: Radius.circular(actionSheet.roundRadius!),
      ),
      child: Container(
//        decoration: BoxDecoration(
//          color: Colors.white,
//            borderRadius: BorderRadius.vertical(
//                top: Radius.circular(actionSheet.round ? 20 : 0)),
//            ),
          child: SafeArea(
              child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          (actionSheet.title != null ||
                  actionSheet.description != null ||
                  actionSheet.customDescription != null)
              ? buildTitleItem(context)
              : Container(),
          ...buildActionSheetItem(context, actionSheet.actions!),
          (actionSheet.cancelText != null)
              ? buildCancelItem(context)
              : Container()
        ],
      ))),
    );
  }
}

class CHFActionSheetItem<T> {
  // 标题
  final String? name;

  // 二级标题
  final String? subName;

  // 选项文字颜色
  final Color? color;

  // 是否为加载状态
  final bool loading;

  // 是否为禁用状态
  final bool disabled;

  // 选项值
  final T? value;

  CHFActionSheetItem({
    this.name,
    this.subName,
    this.color,
    this.loading: false,
    this.disabled: false,
    this.value,
  });
}
